% comuptes simple admixture of NW Eu, SE Eu, AS based
% on Price 2007 snp data, assuming independence of alleles
% replicates dienekes's Eurocalcdna

%note: take off headers, the first line should be the data
disp('Reading deCODEme_scan.csv')
fi=fopen('deCODEme_scan.csv');
decodeme=textscan(fi, '%s %s %n %n %s %s', 'delimiter',',');
fclose(fi);

rsnumber=decodeme{1};
readdirection=char(decodeme{5});
%this is a matrix ndecodeme X 2
allelevalue = char(decodeme{6});
ndecodeme=size(rsnumber,1);

rsnumber_nors=zeros(ndecodeme,1);
for i=1:ndecodeme,
    rsnumber_nors_temp=strrep(rsnumber(i,1),'rs','');
    rsnumber_nors(i,1)=str2num(char(rsnumber_nors_temp));
end;

% this is Price 2007 data
disp('Reading Price 2007 data')
fi=fopen('pricemarkers.txt');
pricemarkers=textscan(fi, 'rs%n %n %n %s %s %n %n %n');
fclose(fi);
rsprice=pricemarkers{1};
refvalue=char(pricemarkers{4});
varvalue=char(pricemarkers{5});
reffreq_nw=pricemarkers{6};
reffreq_se=pricemarkers{7};
reffreq_as=pricemarkers{8};
nprice=size(rsprice,1);

disp('Extracting common rs')
%find which price rs are present in decodeme and in price
k=1;
corresp=zeros(nprice,2);
for i=1:nprice,
    fid=find(rsnumber_nors == rsprice(i,1));
    corresp(i,:)=[rsprice(i,1),-1];
    %insure there is a read for the marker
    % note: we assume that missing values are --
    havegoodread=~strcmp(allelevalue(fid,1:2),'--');
    if (~isempty(fid) & havegoodread),
       corresp(i,:)=[rsprice(i,1),fid(1)];
       cp(k,1:3)=[rsprice(i,1),fid(1),i];
       k=k+1;
    end;
end
nrspresent=k-1;

disp('Creating allele matrix')
%note: matlab prefers dealing with numbers, so we map ACTG to numbers
%      A=1 C=2 G=3 T=4
cacioprdata=zeros(nrspresent,8);
for i=1:nrspresent,
    cacioprdata(i,1)=cp(i,1);
    priceloc=cp(i,3);
    cacioprdata(i,2)=mapACTG(refvalue(priceloc,1),1);
    cacioprdata(i,3)=mapACTG(varvalue(priceloc,1),1);
    %mapping values for decodeme alleles
    rsloc=cp(i,2);
    typemap=1;
    % orientation - has different values, so we need to map A<->T C<->G
    if (strcmp(readdirection(rsloc,1),'-'))
       typemap=2;
    end
    cacioprdata(i,4) = mapACTG(allelevalue(rsloc,1),typemap);
    cacioprdata(i,5) = mapACTG(allelevalue(rsloc,2),typemap); 
    %reference frequencies  
    cacioprdata(i,6) = reffreq_nw(priceloc,1);
    cacioprdata(i,7) = reffreq_se(priceloc,1);
    cacioprdata(i,8) = reffreq_as(priceloc,1);
    cacioprdata(i,9) = typemap;
end;

% it takes a long time to load the matrix, so it is easiest
% to save the data matrix and then simply reload it when
% doing further tests
save priceadmmatrix cacioprdata
disp('Data matrix created')

disp('Computing ML estimates')
%compute ML estimator of admixture of just NW and SE, that is, assuming zero AS
% m NW + (1-m) SE
[m,fval,exitflag]=fminbnd(@(x) logpradmix(x,cacioprdata),0,1);
disp('Admixture results NW-SE')
disp(['  NW = ' num2str(m)])
disp(['  SE = ' num2str(1-m)])

%compute ML estimator of admixture m_1 NW + m_2 SE + m_3 AS
minit=1/3*ones(3,1);
[m3,fval,exitflag]=fmincon(@(x) logpradmix3(x,cacioprdata),minit,[],[], ...
				ones(1,3),1,zeros(3,1),ones(3,1));
disp('Admixture results NW-SE-AS')
disp(['  NW = ' num2str(m3(1,1))])
disp(['  SE = ' num2str(m3(2,1))])
disp(['  AS = ' num2str(m3(3,1))])
